@@ -8,6 +8,7 @@ import ipaddress
 import logging
 import os
 import time
+import pkg_resources
 
 import aiohttp
 import aiohttp.web
@@ -391,3 +392,12 @@ def run_tool_script(coroutine):
             pass
         loop.run_until_complete(loop.shutdown_asyncgens())
         loop.close()
+
+
+def check_required_dependency(req_file: str):
+    """Check the current environment have all dependencies required from a given file.
+
+    Args:
+        req_file (str): The file path of the requirement.txt.
+    """
+    pkg_resources.require(open(req_file).read())
