@@ -11,6 +11,7 @@ from preseem import FakeNetworkMetadataModel, NetworkMetadataReference, FakePres
 from preseem.model_sync import sync_preseem_model, ElementRole, ElementSyncItem
 from preseem_protobuf.model.element_pb2 import ElementStatus
 from preseem_protobuf.model.account_pb2 import AccountType, AccountStatus
+from preseem.source_model import Attachment, Service
 
 Account = namedtuple('Account', ('id', 'type', 'status', 'name', 'first_name', 'last_name'))
 Package = namedtuple('Package', ('id', 'name', 'download_rate', 'upload_rate'))
@@ -766,7 +767,7 @@ class TestModelSync(unittest.TestCase):
     def test_account_update(self):
         """An Account is updated in the model."""
         async def test():
-            accounts = {'acct01': Account('acct01', None, AccountStatus.ACCOUNT_STATUS_INACTIVE, 'Account 1', None, None)}
+            accounts = {'acct01': Account('acct01', None, AccountStatus.ACCOUNT_STATUS_ACTIVE, 'Account 1', None, None)}
             await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], accounts=accounts)
             await self.model.Account.List(company_uuid=self.company.uuid)
             self.assertEqual(len(accounts), 1)
@@ -791,7 +792,7 @@ class TestModelSync(unittest.TestCase):
                     company_uuid=self.company.uuid,
                     source='test_source',
                     source_id='acct01',
-                    status=AccountStatus.ACCOUNT_STATUS_INACTIVE)
+                    status=AccountStatus.ACCOUNT_STATUS_ACTIVE)
 
             # Set it to inactive once and note the inactive timestamp
             await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], accounts={})
@@ -811,7 +812,7 @@ class TestModelSync(unittest.TestCase):
     def test_account_reactivate(self):
         """An inactive account is reactivated in the model."""
         async def test():
-            accounts = {'acct01': Account('acct01', None, AccountStatus.ACCOUNT_STATUS_INACTIVE, 'Account 1', None, None)}
+            accounts = {'acct01': Account('acct01', None, AccountStatus.ACCOUNT_STATUS_ACTIVE, 'Account 1', None, None)}
             await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], accounts=accounts)
             accounts = await self.model.Account.List(company_uuid=self.company.uuid)
             self.assertEqual(len(accounts), 1)
@@ -828,3 +829,294 @@ class TestModelSync(unittest.TestCase):
             self.assertEqual(acct.source, 'test_source')
             self.assertEqual(acct.source_id, 'acct01')
         self._await(test())
+
+    async def checkSourceService(self, ssvc):
+        accounts = {}
+        for account in await self.model.Account.List(company_uuid=self.company.uuid):
+            if account.source_id in accounts:
+                if isinstance(accounts[account.source_id], list):
+                    accounts[account.source_id].append(account)
+                else:
+                    accounts[account.source_id] = [accounts[account.source_id], account]
+            else:
+                accounts[account.source_id] = account
+        packages = {}
+        for package in await self.model.Package.List(company_uuid=self.company.uuid):
+            if package.source_id in packages:
+                if isinstance(packages[package.source_id], list):
+                    packages[package.source_id].append(package)
+                else:
+                    packages[package.source_id] = [packages[package.source_id], package]
+            else:
+                packages[package.source_id] = package
+        services = {x.source_id: x for x in await self.model.Service.List(company_uuid=self.company.uuid)}
+        msvc = services.get(ssvc.id)
+        self.assertIsNotNone(msvc)
+        self.assertEqual(msvc.source_id, ssvc.id)
+        self.assertEqual(msvc.download_rate, ssvc.download_rate)
+        self.assertEqual(msvc.upload_rate, ssvc.upload_rate)
+        self.assertEqual(msvc.subscriber_identifier, ssvc.subscriber_identifier)
+        if ssvc.account_id:
+            account = accounts.get(ssvc.account_id)
+            if isinstance(account, list):  # multiple matching Account objects
+                account = next((x for x in account if x.source==msvc.source), None)
+                if account is None:  # no matching sources
+                    self.assertTrue(ssvc.errors)
+            if account or not ssvc.errors:
+                self.assertIsNotNone(account)
+                self.assertEqual(account.uuid, msvc.account_uuid)
+        if ssvc.package_id:
+            package = packages.get(ssvc.package_id)
+            if isinstance(package, list):  # multiple matching Package objects
+                package = next((x for x in package if x.source==msvc.source), None)
+                if package is None:  # no matching sources
+                    self.assertTrue(ssvc.errors)
+            if package or not ssvc.errors:
+                self.assertIsNotNone(package)
+                self.assertEqual(package.uuid, msvc.package_uuid)
+        if ssvc.attachment:
+            self.assertEqual(ssvc.attachment.pb, msvc.attachment)
+
+    def test_service_create(self):
+        """A Service is created in the model."""
+        async def test():
+            accounts = {'acct01': Account('acct01', None, AccountStatus.ACCOUNT_STATUS_ACTIVE, 'Account 1', None, None)}
+            packages = {'pkg01': Package('pkg01', 'Package 1', 25000, 5000)}
+            service = Service('service01', account_id='acct01', package_id='pkg01', download_rate=100000, upload_rate=20000, subscriber_identifier='account01')
+            service.attachment.cpe_mac_address = '00:01:02:03:04:05'
+            service.attachment.imsi = '310150123456789'
+            service.attachment.cpe_serial_number = 'S12345'
+            service.attachment.cpe_name = '123:456'
+            service.attachment.device_mac_address = '00:0A:95:9D:68:FF'
+            service.attachment.mac_addresses.append('00:01:02:03:04:05')
+            service.attachment.network_prefixes.append('192.0.2.101')
+            service.attachment.network_prefixes.append('3ffe:1900:9876::/48')
+            services = {'service01': service}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], accounts=accounts, packages=packages, services=services)
+            await self.checkSourceService(service)
+        self._await(test())
+
+    def test_service_no_account(self):
+        """If a service references an account that does not exist, account_uuid is not set."""
+        async def test():
+            service = Service('service01', account_id='acct01', download_rate=100000, upload_rate=20000, subscriber_identifier='account01')
+            service.attachment.mac_addresses.append('00:01:02:03:04:05')
+            services = {'service01': service}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], services=services)
+            with self.assertRaises(AssertionError):
+                await self.checkSourceService(service)
+            service.account_id = None
+            await self.checkSourceService(service)
+
+    def test_service_no_package(self):
+        """If a service references a package that does not exist, package_uuid is not set."""
+        async def test():
+            accounts = {'acct01': Account('acct01', None, AccountStatus.ACCOUNT_STATUS_ACTIVE, 'Account 1', None, None)}
+            service = Service('service01', account_id='acct01', package_id='pkg01', download_rate=100000, upload_rate=20000, subscriber_identifier='account01')
+            service.attachment.mac_addresses.append('00:01:02:03:04:05')
+            services = {'service01': service}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], accounts=accounts)
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], services=services)
+            with self.assertRaises(AssertionError):
+                await self.checkSourceService(service)
+            service.package_id = None
+            await self.checkSourceService(service)
+        self._await(test())
+
+    def test_service_account_collision_resolved(self):
+        """If multiple accounts with matching source_id exist, the one with the same source as the service is matched."""
+        async def test():
+            acct1 = await self.model.Account.Create(
+                company_uuid=self.company.uuid,
+                source='test_source',
+                source_id='acct01',
+                status=AccountStatus.ACCOUNT_STATUS_ACTIVE)
+            acct2 = await self.model.Account.Create(
+                company_uuid=self.company.uuid,
+                source='source2',
+                source_id='acct01',
+                status=AccountStatus.ACCOUNT_STATUS_ACTIVE)
+            acct3 = await self.model.Account.Create(
+                company_uuid=self.company.uuid,
+                source='source3',
+                source_id='acct01',
+                status=AccountStatus.ACCOUNT_STATUS_ACTIVE)
+            service = Service('service01', account_id='acct01', download_rate=100000, upload_rate=20000, subscriber_identifier='account01')
+            service.attachment.mac_addresses.append('00:01:02:03:04:05')
+            services = {'service01': service}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], services=services)
+            await self.checkSourceService(service)
+        self._await(test())
+
+    def test_service_account_collision_error(self):
+        """If multiple accounts with matching source_id exist and none match source, an error is flagged on the service."""
+        async def test():
+            acct1 = await self.model.Account.Create(
+                company_uuid=self.company.uuid,
+                source='source1',
+                source_id='acct01',
+                status=AccountStatus.ACCOUNT_STATUS_ACTIVE)
+            acct2 = await self.model.Account.Create(
+                company_uuid=self.company.uuid,
+                source='source2',
+                source_id='acct01',
+                status=AccountStatus.ACCOUNT_STATUS_ACTIVE)
+            acct3 = await self.model.Account.Create(
+                company_uuid=self.company.uuid,
+                source='source3',
+                source_id='acct01',
+                status=AccountStatus.ACCOUNT_STATUS_ACTIVE)
+            service = Service('service01', account_id='acct01', download_rate=100000, upload_rate=20000, subscriber_identifier='account01')
+            service.attachment.mac_addresses.append('00:01:02:03:04:05')
+            services = {'service01': service}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], services=services)
+            await self.checkSourceService(service)
+        self._await(test())
+
+    def test_service_package_collision_resolved(self):
+        """If multiple packages with matching source_id exist, the one with the same source as the service is matched."""
+        async def test():
+            pkg1 = await self.model.Package.Create(
+                company_uuid=self.company.uuid,
+                source='test_source',
+                source_id='pkg01')
+            pkg2 = await self.model.Package.Create(
+                company_uuid=self.company.uuid,
+                source='source2',
+                source_id='pkg01')
+            pkg3 = await self.model.Package.Create(
+                company_uuid=self.company.uuid,
+                source='source3',
+                source_id='pkg01')
+            accounts = {'acct01': Account('acct01', None, AccountStatus.ACCOUNT_STATUS_ACTIVE, 'Account 1', None, None)}
+            service = Service('service01', account_id='acct01', package_id='pkg01', download_rate=100000, upload_rate=20000, subscriber_identifier='account01')
+            service.attachment.mac_addresses.append('00:01:02:03:04:05')
+            services = {'service01': service}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], accounts=accounts, services=services)
+            await self.checkSourceService(service)
+        self._await(test())
+
+    def test_service_package_collision_error(self):
+        """If multiple packages with matching source_id exist and none match source, an error is flagged on the service."""
+        async def test():
+            pkg1 = await self.model.Package.Create(
+                company_uuid=self.company.uuid,
+                source='source1',
+                source_id='pkg01')
+            pkg2 = await self.model.Package.Create(
+                company_uuid=self.company.uuid,
+                source='source2',
+                source_id='pkg01')
+            pkg3 = await self.model.Package.Create(
+                company_uuid=self.company.uuid,
+                source='source3',
+                source_id='pkg01')
+            accounts = {'acct01': Account('acct01', None, AccountStatus.ACCOUNT_STATUS_ACTIVE, 'Account 1', None, None)}
+            service = Service('service01', account_id='acct01', package_id='pkg01', download_rate=100000, upload_rate=20000, subscriber_identifier='account01')
+            service.attachment.mac_addresses.append('00:01:02:03:04:05')
+            services = {'service01': service}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], accounts=accounts, services=services)
+            await self.checkSourceService(service)
+        self._await(test())
+
+    def test_service_update(self):
+        """A Service is updated in the model."""
+        async def test():
+            # create a service
+            accounts = {'acct01': Account('acct01', None, AccountStatus.ACCOUNT_STATUS_ACTIVE, 'Account 1', None, None)}
+            packages = {'pkg01': Package('pkg01', 'Package 1', 25000, 5000)}
+            service = Service('service01', account_id='acct01', package_id='pkg01', download_rate=100000, upload_rate=20000, subscriber_identifier='account01')
+            service.attachment.cpe_mac_address = '00:01:02:03:04:05'
+            service.attachment.imsi = '310150123456789'
+            service.attachment.cpe_serial_number = 'S12345'
+            service.attachment.cpe_name = '123:456'
+            service.attachment.device_mac_address = '00:0A:95:9D:68:FF'
+            service.attachment.mac_addresses.append('00:01:02:03:04:05')
+            service.attachment.network_prefixes.append('192.0.2.101')
+            service.attachment.network_prefixes.append('3ffe:1900:9876::/48')
+            services = {'service01': service}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], accounts=accounts, packages=packages, services=services)
+            await self.checkSourceService(service)
+
+            # change something at each level
+            service.upload_rate = 21000
+            service.subscriber_identifier = None
+            service.attachment.cpe_name = None
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], accounts=accounts, packages=packages, services=services)
+            services = {x.source_id: x for x in await self.model.Service.List(company_uuid=self.company.uuid)}
+            await self.checkSourceService(service)
+        self._await(test())
+
+    def test_service_update_error(self):
+        """Updating a field resolved by the syncer to an invalid value leaves the previous value."""
+        async def test():
+            # create a service
+            accounts = {'acct01': Account('acct01', None, AccountStatus.ACCOUNT_STATUS_ACTIVE, 'Account 1', None, None)}
+            service = Service('service01', account_id='acct01', package_id=None, download_rate=100000, upload_rate=20000, subscriber_identifier='account01')
+            service.attachment.cpe_mac_address = '00:01:02:03:04:05'
+            services = {'service01': service}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], accounts=accounts, services=services)
+            await self.checkSourceService(service)
+
+            service.package_id = 'non-existent-package'
+            service.download_rate = 101000
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], services=services)
+            with self.assertRaises(AssertionError):
+                await self.checkSourceService(service)
+            service.package_id = None
+            await self.checkSourceService(service)
+        self._await(test())
+
+    def test_service_deactivation(self):
+        """A service is deactivated if it is not passed to the syncer."""
+        async def test():
+            # create a service
+            accounts = {'acct01': Account('acct01', None, AccountStatus.ACCOUNT_STATUS_ACTIVE, 'Account 1', None, None)}
+            service = Service('service01', account_id='acct01', package_id=None, download_rate=100000, upload_rate=20000, subscriber_identifier='account01')
+            service.attachment.cpe_mac_address = '00:01:02:03:04:05'
+            services = {'service01': service}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], accounts=accounts, services=services)
+            await self.checkSourceService(service)
+
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], services={})
+            services = {x.source_id: x for x in await self.model.Service.List(company_uuid=self.company.uuid)}
+            await self.checkSourceService(service)
+            msvc = services.get(service.id)
+            self.assertIsNotNone(msvc)
+            self.assertEqual(msvc.inactive, True)
+            t = msvc.inactive_time
+
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], services={})
+            await self.checkSourceService(service)
+            services = {x.source_id: x for x in await self.model.Service.List(company_uuid=self.company.uuid)}
+            msvc = services.get(service.id)
+            self.assertIsNotNone(msvc)
+            self.assertEqual(msvc.inactive_time, t)
+        self._await(test())
+
+    def test_service_reactivation(self):
+        """A service can be reactivated."""
+        async def test():
+            # create a service
+            accounts = {'acct01': Account('acct01', None, AccountStatus.ACCOUNT_STATUS_ACTIVE, 'Account 1', None, None)}
+            service = Service('service01', account_id='acct01', package_id=None, download_rate=100000, upload_rate=20000, subscriber_identifier='account01')
+            service.attachment.cpe_mac_address = '00:01:02:03:04:05'
+            services = {'service01': service}
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], accounts=accounts, services=services)
+            await self.checkSourceService(service)
+
+            # Deactivate it
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], services={})
+            await self.checkSourceService(service)
+            services = {x.source_id: x for x in await self.model.Service.List(company_uuid=self.company.uuid)}
+            msvc = services.get(service.id)
+            self.assertIsNotNone(msvc)
+            self.assertEqual(msvc.inactive, True)
+
+            # Reactivate it
+            await sync_preseem_model(self.model, self.company.uuid, 'test_source', ElementRole.AP, [], services=services)
+            await self.checkSourceService(service)
+            services = {x.source_id: x for x in await self.model.Service.List(company_uuid=self.company.uuid)}
+            msvc = services.get(service.id)
+            self.assertIsNotNone(msvc)
+            self.assertEqual(msvc.inactive, False)
